# Sack
# by Eresse <eresse@eresse.net>

# External Includes
require 'mysql2'

# Internal Includes
require 'sack/database/sanitizer'

# Sack Module
module Sack

	# Connectors Module
	module Connectors

		# MySQL Connector Module:
		# Provides MySQL connectivity for Sack Database.
		module MySQLConnector

			# Open:
			# Opens a connection to a MySQL database.
			# @param [String] conn_string The connection string (host=myServerAddress;database=myDataBase;username=myUsername;password=myPassword;)
			# @return [Object] Database connection
			def self.open conn_string
				Mysql2::Client.new Hash[*(conn_string.split(';').inject([]) { |a, e| a + e.split('=') })]
			end

			# Close:
			# Closes a previously-opened database connection.
			# @param [Object] dbc Database connection
			def self.close dbc
				dbc.close
			end

			# Execute
			# Executes an SQL statement with parameters
			# @param [Object] dbc Database connection
			# @param [String] q Statement
			# @return [Array] Statement results
			def self.exec dbc, q
				dbc.query(q).rows
			end
		end
	end
end
