# Sack
# by Eresse <eresse@eresse.net>

# External Includes
require 'sqlite3'

# Internal Includes
require 'sack/database/sanitizer'

# Sack Module
module Sack

	# Connectors Module
	module Connectors

		# SQLite3 Connector Module:
		# Provides SQLite3 connectivity for Sack Database.
		module SQLite3Connector

			# Open:
			# Opens a connection to an SQLite3 database.
			# @param [String] conn_string The connection string (path to a db file)
			# @return [Object] Database connection
			def self.open conn_string
				SQLite3::Database.open conn_string
			end

			# Close:
			# Closes a previously-opened database connection.
			# @param [Object] dbc Database connection
			def self.close dbc
				dbc.close
			end

			# Execute
			# Executes an SQL statement with parameters
			# @param [Object] dbc Database connection
			# @param [String] q Statement
			# @return [Array] Statement results
			def self.exec dbc, q
				dbc.execute q
			end
		end
	end
end
