# Sack
# by Eresse <eresse@eresse.net>

# Sack Module
module Sack

	# Database Class
	class Database

		# Model Module
		module Model

			# Relationships Module
			module Relationships

				# Class Methods
				module ClassMethods

					# Belongs To:
					# Defines the 'slave' end of any relationship (the one that holds the other entity's ID).
					# @param [Object] options Either a Symbol indicating both the name of the relationship and the target model, or an option Hash defining the relationship - see README
					def belongs_to options

						# Check Options
						if options.is_a? Hash

							# Internalise Options (so we can mess it up)
							options = options.clone

							# Pull Relationship Name & Target Model
							name, target_model_name = options.first
							options.delete name
						else

							# Acquire everything from just a Symbol
							name = options
							target_model_name = name
						end

						# Construct Proxy Method Module
						proxy = Module.new do

							# Proxy Method for Relationship:
							# Allows fetching entities for a given belongs-to relationship.
							define_method name do |db, data|

								# Find Target Model
								target_model = @model_root.const_get target_model_name.to_s.camelcase

								# Fetch Relationship Entity
								target_model.find db, data[name]
							end
						end

						# Extend Model with Proxy
						@model.extend proxy
					end
				end
			end
		end
	end
end
