# Sack
# by Eresse <eresse@eresse.net>

# Sack Module
module Sack

	# Database Class
	class Database

		# Model Module
		module Model

			# Relationships Module
			module Relationships

				# Class Methods
				module ClassMethods

					# Has Many:
					# Defines the 'one' side a one-to-many relationship.
					# @param [Hash] options Options defining the relationship - see README
					def has_many options

						# Internalise Options (so we can mess it up)
						options = options.dclone

						# Pull Relationship Name & Target Model
						name, target_model_name = options.first
						options.delete name

						# Determine Foreign Key (which field in the remote model holds our ID)
						foreign_key = options[:fk] || table_name

						# Register Relationship
						relationships[name] = { target: target_model_name, fk: foreign_key, options: options.dclone }

						# Construct Proxy Method Module
						proxy = Module.new do

							# Proxy Method for Relationship:
							# Allows fetching entities for a given has-many relationship.
							define_method name do |db, data|

								# Find Target Model
								target_model = @model_root.const_get target_model_name.to_s.camelcase

								# Fetch Relationship Entities
								target_model.fetch_by db, foreign_key, data[:id]
							end
						end

						# Extend Model with Proxy
						@model.extend proxy
					end
				end
			end
		end
	end
end
