# Sack
# by Eresse <eresse@eresse.net>

# External Includes
require 'aromat'

# Sack Module
module Sack

	# Database Class
	class Database

		# Schema Module:
		# Provides utilities for manipulating database schema.
		module Schema

			# Load from Module:
			# Constructs a database schema from a given data model module (_mod_).
			# @param [Module] mod Data model module, containing entity modules
			# @return [Hash] The complete database schema
			def self.from_module mod

				# Run through Sub Modules
				Hash[*(mod.constants
					.collect { |c| mod.const_get c }
					.select { |c| c.is_a? Module }
					.inject([]) { |a, e| (a << e.table_name) << e.field_schema }
				)]
			end
		end
	end
end
